package io.tencent.trtc.ss.impl;

import android.Manifest;
import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.SurfaceTexture;
import android.hardware.display.DisplayManager;
import android.hardware.display.VirtualDisplay;
import android.media.AudioAttributes;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.projection.MediaProjection;
import android.media.projection.MediaProjectionManager;
import android.opengl.GLES20;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.util.Log;
import android.view.Surface;
import android.view.Window;
import android.media.AudioPlaybackCaptureConfiguration;

import androidx.annotation.RequiresApi;

import com.tencent.liteav.demo.common.customcapture.opengl.GPUImageFilter;
import com.tencent.liteav.demo.common.customcapture.opengl.GPUImageFilterGroup;
import com.tencent.liteav.demo.common.customcapture.opengl.OesInputFilter;
import com.tencent.liteav.demo.common.customcapture.opengl.OpenGlUtils;
import com.tencent.liteav.demo.common.customcapture.opengl.Rotation;
import com.tencent.liteav.demo.common.customcapture.structs.FrameBuffer;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.ref.WeakReference;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.concurrent.atomic.AtomicInteger;

import io.tencent.trtc.ss.gles.GLRender;
import io.tencent.trtc.ss.gles.GlUtil;
import io.tencent.trtc.ss.gles.ImgTexFormat;
import io.tencent.trtc.ss.gles.ImgTexFrame;
import io.tencent.trtc.ss.gles.SrcConnector;

/**
 * capture video frames from screen
 */
@TargetApi(Build.VERSION_CODES.LOLLIPOP)
public class ScreenCapture implements SurfaceTexture.OnFrameAvailableListener {

    private static final boolean DEBUG_ENABLED = true;

    private static final String TAG = ScreenCapture.class.getSimpleName();

    public static final int MEDIA_PROJECTION_REQUEST_CODE = 1001;

    private Context mContext;
    private OnScreenCaptureListener mOnScreenCaptureListener;
    public MediaProjectionManager mMediaProjectManager;  // mMediaProjectionManager
    private MediaProjection mMediaProjection;  // mMediaProjection
    private VirtualDisplay mVirtualDisplay;  // mVirtualDisplay

    private FrameBuffer mFrameBuffer;
    private OesInputFilter mOesInputFilter;
    private GPUImageFilterGroup mGpuImageFilterGroup;

    private  FloatBuffer mGLCubeBuffer;
    private  FloatBuffer mGLTextureBuffer;

    private int mWidth = 1280;  // mWidth
    private int mHeight = 720;  // mHeight

    public final static int SCREEN_STATE_IDLE = 0;
    public final static int SCREEN_STATE_INITIALIZING = 1;
    public final static int SCREEN_STATE_INITIALIZED = 2;
    public final static int SCREEN_STATE_STOPPING = 3;
    public final static int SCREEN_STATE_CAPTURING = 4;

    public final static int SCREEN_ERROR_SYSTEM_UNSUPPORTED = -1;
    public final static int SCREEN_ERROR_PERMISSION_DENIED = -2;

    public final static int SCREEN_RECORD_STARTED = 4;
    public final static int SCREEN_RECORD_FAILED = 5;

    private final static int MSG_SCREEN_START_SCREEN_ACTIVITY = 1;
    private final static int MSG_SCREEN_INIT_PROJECTION = 2;
    private final static int MSG_SCREEN_START = 3;
    private final static int MSG_AUDIO_START = 4;
    private final static int MSG_SCREEN_RELEASE = 5;
    private final static int MSG_AUDIO_RELEASE = 6;
    private final static int MSG_SCREEN_QUIT = 7;

    private final static int RELEASE_SCREEN_THREAD = 1;

    public final static int AUDIO_SAMPLE_RATE = 48000;
    public final static int AUDIO_CHANNEL_NUM = 2;
    public final static int AUDIO_SOURCE_NUM = 1;
    public final static int NUM_SAMPLES_PER_READ = 10240;
    public final static int BYTES_PER_SAMPLE = 2; // 2 bytes since we hardcoded the PCM 16-bit format
    public final static int BUFFER_SIZE_IN_BYTES = NUM_SAMPLES_PER_READ * BYTES_PER_SAMPLE;

    private AtomicInteger mState;

    private GLRender mGLRender;
    private int mTextureId;
    private Surface mSurface;
    private SurfaceTexture mSurfaceTexture;
    private boolean mTexInited = false;
    private ImgTexFormat mImgTexFormat;

    private Handler mMainHandler;
    private HandlerThread mScreenSetupThread;
    private Handler mScreenSetupHandler;
    private int mScreenDensity;

    private DataOutputStream audioWriter = null;

    public void openfile() {
        File sampleDir = new File(mContext.getExternalFilesDir(Environment.DIRECTORY_MUSIC), "/pcm");
        if (!sampleDir.exists()) {
            sampleDir.mkdirs();
        }
        String fileName = "Record-" + new SimpleDateFormat("yyyy-MM-dd hh-mm-ss").format(new Date()) + ".pcm";
        String filePath = sampleDir +"/"+ fileName;
        try {
            File file = new File(filePath);
            OutputStream os = new FileOutputStream(file);
            BufferedOutputStream bos = new BufferedOutputStream(os);
            audioWriter = new DataOutputStream(bos);
        } catch (FileNotFoundException e) {
            Log.e("SUMN",e.getLocalizedMessage());
            e.printStackTrace();

        }
    }

    // fill extra frame
    private Runnable mFillFrameRunnable;
    private Runnable mAudioRecordRunnable = new Runnable() {
        @Override
        public void run() {
            if (audioRecord == null) {
                Log.e(TAG, "AudioRecord is null!");
                return;
            }
            byte[] capturedAudioSamples = new byte[BUFFER_SIZE_IN_BYTES];
            audioRecord.read(capturedAudioSamples, 0, BUFFER_SIZE_IN_BYTES);
            int size = capturedAudioSamples.length;
            ByteBuffer frame = ByteBuffer.allocateDirect(size);
            frame.order(ByteOrder.LITTLE_ENDIAN);
            frame.put(capturedAudioSamples, 0, size);
            frame.flip();
            try {
                // // writes the data to file from buffer
                // // stores the voice buffer
                if (audioWriter != null) {
                    audioWriter.write(capturedAudioSamples, 0, size);
                }
            } catch (IOException e) {
                e.printStackTrace();
                Log.i("tag", "record file error:" + e.getMessage());
            }
            audioFrameListener.onFrame(frame, size);
            mScreenSetupHandler.postDelayed(mAudioRecordRunnable, 10);
        }
    };

    private final static boolean TRACE = true;
    // Performance trace
    private long mLastTraceTime;
    private long mFrameDrawed;
    private AudioRecord audioRecord;
    private AudioFrameListener audioFrameListener;

    public void setAudioFrameListener(AudioFrameListener audioFrameListener){
        this.audioFrameListener = audioFrameListener;
    }

    /**
     * Source pin transfer ImgTexFrame, used for gpu path and preview
     */
    public SrcConnector<ImgTexFrame> mImgTexSrcConnector;

    public ScreenCapture(Context context, GLRender render, int density) {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            throw new RuntimeException("Need API level " + Build.VERSION_CODES.LOLLIPOP);
        }

        if (context == null || render == null) {
            throw new IllegalArgumentException("the context or render must be not null");
        }

        mContext = context;
        mGLRender = render;
        mScreenDensity = density;

        mGLRender.addListener(mGLRenderListener);
        mImgTexSrcConnector = new SrcConnector<>();
        mMainHandler = new MainHandler(this);
        mState = new AtomicInteger(SCREEN_STATE_IDLE);
        mFillFrameRunnable = new Runnable() {
            @Override
            public void run() {
                if (mState.get() == SCREEN_STATE_CAPTURING) {
                    mGLRender.requestRender();
                    mMainHandler.postDelayed(mFillFrameRunnable, 500);
                }
            }
        };

        initScreenSetupThread();
    }

    /**
     * Start screen record.<br/>
     * Can only be called on mState IDLE.
     */
    public boolean start() {
        if (DEBUG_ENABLED) {
            Log.d(TAG, "start");
        }

        if (mState.get() != SCREEN_STATE_IDLE) {
            return false;
        }

        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            Message msg = mMainHandler.obtainMessage(SCREEN_RECORD_FAILED, SCREEN_ERROR_SYSTEM_UNSUPPORTED, 0);
            mMainHandler.sendMessage(msg);
            return false;
        }

        mState.set(SCREEN_STATE_INITIALIZING);
        mScreenSetupHandler.removeMessages(MSG_SCREEN_START_SCREEN_ACTIVITY);
        mScreenSetupHandler.sendEmptyMessage(MSG_SCREEN_START_SCREEN_ACTIVITY);
        return true;
    }

    /**
     * stop screen record
     */
    public void stop() {
        if (DEBUG_ENABLED) {
            Log.d(TAG, "stop");
        }

        if (mState.get() == SCREEN_STATE_IDLE) {
            return;
        }

        // stop fill frame
        mMainHandler.removeCallbacks(mFillFrameRunnable);

        Message msg = new Message();
        msg.what = MSG_SCREEN_RELEASE;
        msg.arg1 = ~RELEASE_SCREEN_THREAD;

        mState.set(SCREEN_STATE_STOPPING);
        mScreenSetupHandler.removeMessages(MSG_SCREEN_RELEASE);
        mScreenSetupHandler.sendMessage(msg);
        mScreenSetupHandler.removeMessages(MSG_AUDIO_START);
        mScreenSetupHandler.removeCallbacks(mAudioRecordRunnable);
        mScreenSetupHandler.sendEmptyMessage(MSG_AUDIO_RELEASE);
    }

    public void release() {
        // stop fill frame
        if (mMainHandler != null) {
            mMainHandler.removeCallbacks(mFillFrameRunnable);
        }

        if (mState.get() == SCREEN_STATE_IDLE) {
            mScreenSetupHandler.removeMessages(MSG_SCREEN_QUIT);
            mScreenSetupHandler.sendEmptyMessage(MSG_SCREEN_QUIT);
            quitThread();
            return;
        }

        Message msg = new Message();
        msg.what = MSG_SCREEN_RELEASE;
        msg.arg1 = RELEASE_SCREEN_THREAD;

        mState.set(SCREEN_STATE_STOPPING);
        mScreenSetupHandler.removeMessages(MSG_SCREEN_RELEASE);
        mScreenSetupHandler.sendMessage(msg);

        quitThread();
    }

    /**
     * screen status changed listener
     *
     * @param listener
     */
    public void setOnScreenCaptureListener(OnScreenCaptureListener listener) {
        mOnScreenCaptureListener = listener;
    }

    @Override
    public void onFrameAvailable(SurfaceTexture st) {
        if (mState.get() != SCREEN_STATE_CAPTURING) {
            return;
        }
        mGLRender.requestRender();
        if (mMainHandler != null) {
            mMainHandler.removeCallbacks(mFillFrameRunnable);
            mMainHandler.postDelayed(mFillFrameRunnable, 100);
        }
    }

    private void initTexFormat() {
        mImgTexFormat = new ImgTexFormat(ImgTexFormat.COLOR_FORMAT_EXTERNAL_OES, mWidth, mHeight);
        mImgTexSrcConnector.onFormatChanged(mImgTexFormat);
    }

    @RequiresApi(api = Build.VERSION_CODES.Q)
    public final void initProjection(int requestCode, int resultCode, Intent intent) {
        if (DEBUG_ENABLED) {
            Log.d(TAG, "initProjection");
        }

        if (requestCode != MEDIA_PROJECTION_REQUEST_CODE) {
            if (DEBUG_ENABLED) {
                Log.d(TAG, "Unknown request code: " + requestCode);
            }
        } else if (resultCode != Activity.RESULT_OK) {
            Log.e(TAG, "Screen Cast Permission Denied, resultCode: " + resultCode);
            Message msg = mMainHandler.obtainMessage(SCREEN_RECORD_FAILED,
                    SCREEN_ERROR_PERMISSION_DENIED, 0);
            mMainHandler.sendMessage(msg);
            stop();
        } else {
            // get media projection and virtual display
            mMediaProjection = mMediaProjectManager.getMediaProjection(resultCode, intent);

            if (mSurface != null) {
                startScreenCapture();
            } else {
                mState.set(SCREEN_STATE_INITIALIZED);
            }
            startAudioCapture();
        }
    }

    public void doDrawFrame() {
        long pts = System.nanoTime() / 1000 / 1000;

        try {
            mSurfaceTexture.updateTexImage();
        } catch (Exception e) {
            Log.e(TAG, "updateTexImage failed, ignore"+e.getMessage());
            return;
        }

        if (!mTexInited) {
            mTexInited = true;
            initTexFormat();
        }

        float[] texMatrix = new float[16];
        mSurfaceTexture.getTransformMatrix(texMatrix);

        mOesInputFilter.setTexutreTransform(texMatrix);
        GLES20.glViewport(0, 0, mWidth, mHeight);
        mGpuImageFilterGroup.draw(mTextureId, mFrameBuffer.getFrameBufferId(), mGLCubeBuffer, mGLTextureBuffer);

        ImgTexFrame frame = new ImgTexFrame(mImgTexFormat, mFrameBuffer.getTextureId(), texMatrix, pts);

        try {
            mImgTexSrcConnector.onFrameAvailable(frame,null);
        } catch (Exception e) {
            e.printStackTrace();
            Log.e(TAG, "Draw frame failed, ignore");
        }

        if (TRACE) {
            mFrameDrawed++;
            long tm = System.currentTimeMillis();
            long tmDiff = tm - mLastTraceTime;
            if (tmDiff >= 5000) {
                float fps = mFrameDrawed * 1000.f / tmDiff;
                Log.d(TAG, "screen fps: " + String.format(Locale.getDefault(), "%.2f", fps));
                mFrameDrawed = 0;
                mLastTraceTime = tm;
            }
        }
    }

    private GLRender.GLRenderListener mGLRenderListener = new GLRender.GLRenderListener() {
        @Override
        public void onReady() {
            Log.d(TAG, "onReady");
        }

        @Override
        public void onSizeChanged(int width, int height) {
            Log.d(TAG, "onSizeChanged : " + width + "*" + height);
            mWidth = width;
            mHeight = height;

            mTexInited = false;

            if (mVirtualDisplay != null) {
                mVirtualDisplay.release();
                mVirtualDisplay = null;
            }

            mTextureId = GlUtil.createOESTextureObject();
            if (mSurfaceTexture != null) {
                mSurfaceTexture.release();
            }

            if (mSurface != null) {
                mSurface.release();
            }
            mSurfaceTexture = new SurfaceTexture(mTextureId);
            mSurfaceTexture.setDefaultBufferSize(mWidth, mHeight);
            mSurface = new Surface(mSurfaceTexture);

            mSurfaceTexture.setOnFrameAvailableListener(ScreenCapture.this);

            if (mState.get() >= SCREEN_STATE_INITIALIZED && mVirtualDisplay == null) {
                mScreenSetupHandler.removeMessages(MSG_SCREEN_START);
                mScreenSetupHandler.sendEmptyMessage(MSG_SCREEN_START);
            }

            mGLCubeBuffer = OpenGlUtils.createNormalCubeVerticesBuffer();
            mGLTextureBuffer = OpenGlUtils.createTextureCoordsBuffer(Rotation.NORMAL, false, false);

            // 创建一个FrameBuffer，作为输出给到外面（外面不能异步使用）
            mFrameBuffer = new FrameBuffer(mWidth, mHeight);
            mFrameBuffer.initialize();

            mGpuImageFilterGroup = new GPUImageFilterGroup();
            mOesInputFilter = new OesInputFilter();
            mGpuImageFilterGroup.addFilter(mOesInputFilter);
            mGpuImageFilterGroup.addFilter(new GPUImageFilter(true));
            mGpuImageFilterGroup.init();
            mGpuImageFilterGroup.onOutputSizeChanged(mWidth, mHeight);
        }

        @Override
        public void onDrawFrame() {
			doDrawFrame();
        }

        @Override
        public void onReleased() {

        }
    };

    private void startScreenCapture() {
        mVirtualDisplay = mMediaProjection.createVirtualDisplay("ScreenCapture",
                mWidth, mHeight, mScreenDensity, DisplayManager.VIRTUAL_DISPLAY_FLAG_PUBLIC, mSurface,
                null, null);

        mState.set(SCREEN_STATE_CAPTURING);
        Message msg = mMainHandler.obtainMessage(SCREEN_RECORD_STARTED, 0, 0);
        mMainHandler.sendMessage(msg);
    }

    @RequiresApi(api = Build.VERSION_CODES.Q)
    private void startAudioCapture() {
        AudioPlaybackCaptureConfiguration config = new AudioPlaybackCaptureConfiguration.Builder(mMediaProjection).addMatchingUsage(AudioAttributes.USAGE_MEDIA).build();
        AudioFormat audioFormat = new AudioFormat.Builder()
                .setEncoding(AudioFormat.ENCODING_PCM_16BIT)
                .setSampleRate(AUDIO_SAMPLE_RATE)
                .setChannelMask(AudioFormat.CHANNEL_IN_MONO)
                .build();
        audioRecord = new AudioRecord.Builder()
                .setAudioFormat(audioFormat)
                // For optimal performance, the buffer size
                // can be optionally specified to store audio samples.
                // If the value is not specified,
                // uses a single frame and lets the
                // native code figure out the minimum buffer size.
                .setBufferSizeInBytes(BUFFER_SIZE_IN_BYTES)
                .setAudioPlaybackCaptureConfig(config)
                .build();
        openfile();
        mScreenSetupHandler.sendEmptyMessageDelayed(MSG_AUDIO_START, 3000);
        mScreenSetupHandler.postDelayed(mAudioRecordRunnable,5000);
    }

    private static class MainHandler extends Handler {
        private final WeakReference<ScreenCapture> weakCapture;

        public MainHandler(ScreenCapture screenCapture) {
            super();
            this.weakCapture = new WeakReference<>(screenCapture);
        }

        @Override
        public void handleMessage(Message msg) {
            ScreenCapture screenCapture = weakCapture.get();
            if (screenCapture == null) {
                return;
            }
            switch (msg.what) {
                case SCREEN_RECORD_STARTED:
                    if (screenCapture.mOnScreenCaptureListener != null) {
                        screenCapture.mOnScreenCaptureListener.onStarted();
                    }
                    break;
                case SCREEN_RECORD_FAILED:
                    if (screenCapture.mOnScreenCaptureListener != null) {
                        screenCapture.mOnScreenCaptureListener.onError(msg.arg1);
                    }
                    break;
                default:
                    break;

            }
        }
    }

    private void initScreenSetupThread() {
        mScreenSetupThread = new HandlerThread("screen_setup_thread", Thread.NORM_PRIORITY);
        mScreenSetupThread.start();
        mScreenSetupHandler = new Handler(mScreenSetupThread.getLooper()) {
            @Override
            public void handleMessage(Message msg) {
                switch (msg.what) {
                    case MSG_SCREEN_START_SCREEN_ACTIVITY: {
                        doScreenSetup();
                        break;
                    }
                    case MSG_SCREEN_INIT_PROJECTION: {
                        initProjection(msg.arg1, msg.arg2, mProjectionIntent);
                        break;
                    }
                    case MSG_SCREEN_START: {
                        startScreenCapture();
                        break;
                    }
                    case MSG_AUDIO_START: {
                        if (audioRecord != null) {
                            audioRecord.startRecording();
                        }
                        break;
                    }
                    case MSG_SCREEN_RELEASE: {
                        doScreenRelease(msg.arg1);
                        break;
                    }
                    case MSG_AUDIO_RELEASE: {
                        if (audioRecord != null) {
                            audioRecord.stop();
                            audioRecord = null;
                        }
                        break;
                    }
                    case MSG_SCREEN_QUIT: {
                        mScreenSetupThread.quit();
                        break;
                    }
                }
            }
        };
    }

    private void quitThread() {
        try {
            mScreenSetupThread.join();
        } catch (InterruptedException e) {
            Log.d(TAG, "quitThread " + Log.getStackTraceString(e));
        } finally {
            mScreenSetupThread = null;
        }

        if (mMainHandler != null) {
            mMainHandler.removeCallbacksAndMessages(null);
            mMainHandler = null;
        }
    }

    private void doScreenSetup() {
        if (DEBUG_ENABLED) {
            Log.d(TAG, "doScreenSetup");
        }

        if (mMediaProjectManager == null) {
            mMediaProjectManager = (MediaProjectionManager) mContext.getSystemService(
                    Context.MEDIA_PROJECTION_SERVICE);
        }
        Log.d(TAG, "startActivity ScreenCapture");
        Message msg = new Message();
        msg.what = MSG_SCREEN_INIT_PROJECTION;
        msg.arg1 = MEDIA_PROJECTION_REQUEST_CODE;
        msg.arg2 = Activity.RESULT_OK;
        mScreenSetupHandler.removeMessages(MSG_SCREEN_INIT_PROJECTION);
        mScreenSetupHandler.sendMessage(msg);
    }

    private void doScreenRelease(int isQuit) {
        if (DEBUG_ENABLED) {
            Log.d(TAG, "doScreenRelease");
        }

        mState.set(SCREEN_STATE_IDLE);

        if (mVirtualDisplay != null) {
            mVirtualDisplay.release();
        }

        if (mMediaProjection != null) {
            mMediaProjection.stop();
        }

        mVirtualDisplay = null;
        mMediaProjection = null;

        if (isQuit == RELEASE_SCREEN_THREAD) {
            mScreenSetupHandler.sendEmptyMessage(MSG_SCREEN_QUIT);
        }
    }

    public static Intent mProjectionIntent;

    public interface OnScreenCaptureListener {

        /**
         * Notify screen capture started.
         */
        void onStarted();

        /**
         * Notify error occurred while camera capturing.
         *
         * @param err err code.
         * @see #SCREEN_ERROR_SYSTEM_UNSUPPORTED
         * @see #SCREEN_ERROR_PERMISSION_DENIED
         */
        void onError(int err);
    }

    public interface AudioFrameListener {

        void onFrame(ByteBuffer buffer, long length);
    }

}

