package io.tencent.trtc.ss.impl;

import android.app.Notification;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;
import android.os.Process;
import android.os.RemoteCallbackList;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Surface;
import android.view.WindowManager;

import androidx.core.app.NotificationCompat;

import com.tencent.trtc.TRTCCloud;
import com.tencent.trtc.TRTCCloudDef;

import java.nio.ByteBuffer;
import io.tencent.trtc.ss.Constant;
import io.tencent.trtc.ss.R;
import io.tencent.trtc.ss.aidl.INotification;
import io.tencent.trtc.ss.aidl.IScreenSharing;
import io.tencent.trtc.ss.gles.GLRender;
import io.tencent.trtc.ss.gles.ImgTexFrame;
import io.tencent.trtc.ss.gles.SinkConnector;


public class ScreenSharingService extends Service {

    private static final String LOG_TAG = ScreenSharingService.class.getSimpleName();

    private ScreenCapture mScreenCapture;
    private GLRender mScreenGLRender;
    private TRTCCloud mTRTCCloud;
    private String mChannelId;
//    private RtcConnection mRtcConnection = new RtcConnection();
    private boolean mIsLandSpace = false;
    private Context mContext;

    private RemoteCallbackList<INotification> mCallbacks
            = new RemoteCallbackList<INotification>();

    private final IScreenSharing.Stub mBinder = new IScreenSharing.Stub() {
        public void registerCallback(INotification cb) {
            if (cb != null) mCallbacks.register(cb);
        }

        public void unregisterCallback(INotification cb) {
            if (cb != null) mCallbacks.unregister(cb);
        }

        public void startShare(String token, String channelName, int uid) {
            startCapture(token, channelName, uid);
        }

        public void stopShare() {
            stopCapture();
        }

        public void renewToken(String token) {
            refreshToken(token);
        }

    };

    private void initModules() {
        WindowManager wm = (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
        DisplayMetrics metrics = new DisplayMetrics();
        wm.getDefaultDisplay().getMetrics(metrics);

        if (mScreenGLRender == null) {
            mScreenGLRender = new GLRender();
        }

        if (mScreenCapture == null) {
            mScreenCapture = new ScreenCapture(mContext, mScreenGLRender, metrics.densityDpi);
        }

        mScreenCapture.mImgTexSrcConnector.connect(new SinkConnector<ImgTexFrame>() {
            @Override
            public void onFormatChanged(Object obj) {
                Log.d(LOG_TAG, "onFormatChanged " + obj.toString());
            }

            @Override
            public void onFrameAvailable(ImgTexFrame frame, ByteBuffer byteBuffer) {

                if (mTRTCCloud == null) {
                    return;
                }
                //Log.e(LOG_TAG, "xzm: onFrameAvailable " + frame.toString() + " " + frame.pts);

                // 将视频帧通过纹理方式塞给SDK
                TRTCCloudDef.TRTCVideoFrame videoFrame = new TRTCCloudDef.TRTCVideoFrame();
                videoFrame.texture = new TRTCCloudDef.TRTCTexture();
                videoFrame.texture.textureId = frame.mTextureId;
                videoFrame.texture.eglContext14 = mScreenGLRender.getEGLContext();
                videoFrame.width = frame.mFormat.mWidth;
                videoFrame.height = frame.mFormat.mHeight;
                videoFrame.pixelFormat = TRTCCloudDef.TRTC_VIDEO_PIXEL_FORMAT_Texture_2D;
                videoFrame.bufferType = TRTCCloudDef.TRTC_VIDEO_BUFFER_TYPE_TEXTURE;
//                mTRTCCloud.sendCustomVideoData(videoFrame);
//
//                TRTCCloudDef.TRTCVideoFrame trtcFrame = new TRTCCloudDef.TRTCVideoFrame();
//                trtcFrame.bufferType
//                VideoFrame.TextureBuffer buffer = new TextureBuffer(
//                        mScreenGLRender.getEGLContext(),
//                        frame.mFormat.mWidth,
//                        frame.mFormat.mHeight,
//                        VideoFrame.TextureBuffer.Type.OES,
//                        frame.mTextureId,
//                        RendererCommon.convertMatrixToAndroidGraphicsMatrix(frame.mTexMatrix),
//                        null,
//                        null,
//                        null);
//                VideoFrame videoFrame = new VideoFrame(buffer, 0, System.nanoTime());
                mTRTCCloud.sendCustomVideoData(TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB, videoFrame);
            }

            @Override
            public void setSurface(Surface surface) {
                Log.i("xzm","setSurface");
            }
        });

        mScreenCapture.setOnScreenCaptureListener(new ScreenCapture.OnScreenCaptureListener() {
            @Override
            public void onStarted() {
                Log.d(LOG_TAG, "Screen Record Started");
            }

            @Override
            public void onError(int err) {
                Log.d(LOG_TAG, "onError " + err);
                switch (err) {
                    case ScreenCapture.SCREEN_ERROR_SYSTEM_UNSUPPORTED:
                        break;
                    case ScreenCapture.SCREEN_ERROR_PERMISSION_DENIED:
                        break;
                }
            }
        });

        mScreenCapture.setAudioFrameListener(new ScreenCapture.AudioFrameListener() {
            @Override
            public void onFrame(ByteBuffer buffer, long length) {
                Log.d("tag", "onFrame");
                if(mTRTCCloud != null){
                    TRTCCloudDef.TRTCAudioFrame frame = new TRTCCloudDef.TRTCAudioFrame();
                    frame.data = new byte[(int) length];
                    buffer.position(0);
                    buffer.get(frame.data);
                    frame.timestamp = 0;
                    frame.sampleRate = 48000;
                    frame.channel = 1;

                    mTRTCCloud.mixExternalAudioFrame(frame);
                }
            }
        });

        int screenWidth = wm.getDefaultDisplay().getWidth();
        int screenHeight = wm.getDefaultDisplay().getHeight();
        if ((mIsLandSpace && screenWidth < screenHeight) ||
                (!mIsLandSpace) && screenWidth > screenHeight) {
            screenWidth = wm.getDefaultDisplay().getHeight();
            screenHeight = wm.getDefaultDisplay().getWidth();
        }

        setOffscreenPreview(screenWidth, screenHeight);
    }

    private void deInitModules() {
        if (mScreenCapture != null) {
            mScreenCapture.release();
            mScreenCapture = null;
        }

        if (mScreenGLRender != null) {
            mScreenGLRender.quit();
            mScreenGLRender = null;
        }
    }

    public void setSurface(Surface surface) {
//        mRtcEngine.setupInputSurface2(surface);
    }

    /**
     * Set offscreen preview.
     *
     * @param width  offscreen width
     * @param height offscreen height
     * @throws IllegalArgumentException
     */
    public void setOffscreenPreview(int width, int height) throws IllegalArgumentException {
        if (width <= 0 || height <= 0) {
            throw new IllegalArgumentException("Invalid offscreen resolution");
        }

        mScreenGLRender.init(width, height);
    }

    private void startCapture(String token, String channelName, int uid) {
        Log.i("xzm", "startCapture");
        initModules();
        mChannelId = channelName;
//        if (joinChannel(token, channelName, uid) != Constants.ERR_OK){
//            Log.e("xzm", "joinChannel failed");
//            return;
//        }
        startForeground(55431, getForeNotification());
        mScreenCapture.start();
    }

    public Notification getForeNotification() {
        Notification notification;
        String eventTitle = getResources().getString(R.string.app_name);
//        String eventTitle = "";
        NotificationCompat.Builder builder = new NotificationCompat.Builder(this, NotificationHelper.generateChannelId(getApplication(), 55431))
                .setContentTitle(eventTitle)
                .setContentText(eventTitle);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
            builder.setColor(getResources().getColor(android.R.color.black));
        notification = builder.build();
        notification.flags |= Notification.FLAG_ONGOING_EVENT;

        return notification;
    }

    private void stopCapture() {
        stopForeground(true);
        if (mScreenCapture != null) {
            mScreenCapture.stop();
        }
//        mRtcEngine.leaveChannelEx(mChannelId, mRtcConnection);
        deInitModules();
    }

    private void refreshToken(String token) {
//        if (mRtcEngine != null) {
//            mRtcEngine.renewToken(token);
//        } else {
//            Log.e(LOG_TAG, "rtc engine is null");
//        }
    }

    @Override
    public void onCreate() {
        mContext = getApplicationContext();
        mTRTCCloud = TRTCCloud.sharedInstance(getApplicationContext());
    }

    @Override
    public IBinder onBind(Intent intent) {
        return mBinder;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        deInitModules();
    }

}
